# -*- coding: utf-8 -*-

# CC-Metrics -- Call Center Metrics from Asterisk PBX
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2014-2015 Enercoop, Easter-eggs
# https://gitlab.easter-eggs.com/ee/cc-metrics
#
# This file is part of CC-Metrics.
#
# CC-Metrics is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# CC-Metrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


""" Pyramid initialization """

from pyramid.config import Configurator
from pyramid.session import UnencryptedCookieSessionFactoryConfig

from ccmetrics.extensions import initialize_lookup
from ccmetrics.models import initialize_model


class AggregatePredicate(object):
    """
    Custom aggregate predicate
    """

    def __init__(self, name, config):
        self.name = name

    def text(self):
        return self.name

    phash = text

    def __call__(self, context, request):
        matchdict = context['match']
        return matchdict[self.name] in ('avg', 'max', 'min', 'sum')


class PagePredicate(object):
    """
    Custom page predicate
    """

    def __init__(self, name, config):
        self.name = name

    def text(self):
        return self.name

    phash = text

    def __call__(self, context, request):
        matchdict = context['match']
        return matchdict[self.name] in ('abandoned', 'answered', 'charts', 'incoming', 'internal', 'missed', 'outgoing', 'queue')


def main(global_config, **settings):
    """
    This function returns a Pyramid WSGI application.
    """

    # Model setup
    initialize_model(settings)

    # Lookup setup
    initialize_lookup(settings)

    # Session setup
    session_key = global_config.get('pyramid.session.key', 'the-secret-session-key')
    session_factory = UnencryptedCookieSessionFactoryConfig(session_key)

    config = Configurator(
        session_factory=session_factory,
        settings=settings,
    )

    # I18n setup
    config.set_locale_negotiator('pyramid_helpers.i18n.header_locale_negotiator')
    config.add_translation_dirs('ccmetrics:locale/')

    config.add_subscriber('pyramid_helpers.i18n.add_renderer_globals', 'pyramid.events.BeforeRender')
    config.add_subscriber('pyramid_helpers.i18n.add_localizer', 'pyramid.events.NewRequest')

    # Forms setup
    config.add_subscriber('pyramid_helpers.forms.on_before_renderer', 'pyramid.events.BeforeRender')
    config.add_subscriber('pyramid_helpers.forms.on_new_request', 'pyramid.events.NewRequest')

    # Add CSV renderer
    config.add_renderer('csv', 'pyramid_helpers.renderers.csv_renderer_factory')

    # Replace JSON renderer
    config.add_renderer('json', 'pyramid_helpers.renderers.json_renderer_factory')

    # Static route
    config.add_static_view('static', 'static', cache_max_age=3600)

    # Custom predicates
    config.add_route_predicate('page_predicate', PagePredicate)
    config.add_route_predicate('aggregate_predicate', AggregatePredicate)

    # Front
    config.add_route('index', '/')
    config.add_route('dashboard', '/dashboard/:page', page_predicate='page')
    config.add_route('incoming', '/incoming/:page', page_predicate='page')
    config.add_route('outgoing', '/outgoing/:page', page_predicate='page')
    config.add_route('queue', '/queue/:page', page_predicate='page')

    #
    # API v1.0
    #

    # Incoming calls
    config.add_route('api.incoming.answered', '/api/1.0/incoming/answered')
    config.add_route('api.incoming.answered-range', '/api/1.0/incoming/answered/range')
    config.add_route('api.incoming.answered-slice', '/api/1.0/incoming/answered/slice')
    config.add_route('api.incoming.answered-table', '/api/1.0/incoming/answered/table')
    config.add_route('api.incoming.duration', '/api/1.0/incoming/duration/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.incoming.duration-range', '/api/1.0/incoming/duration/range')
    config.add_route('api.incoming.duration-slice', '/api/1.0/incoming/duration/slice/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.incoming.missed', '/api/1.0/incoming/missed')
    config.add_route('api.incoming.missed-range', '/api/1.0/incoming/missed/range')
    config.add_route('api.incoming.missed-slice', '/api/1.0/incoming/missed/slice')
    config.add_route('api.incoming.missed-table', '/api/1.0/incoming/missed/table')
    config.add_route('api.incoming.ratio', '/api/1.0/incoming/ratio')
    config.add_route('api.incoming.ratio-range', '/api/1.0/incoming/ratio/range')
    config.add_route('api.incoming.ratio-slice', '/api/1.0/incoming/ratio/slice')
    config.add_route('api.incoming.stats', '/api/1.0/incoming/stats')

    config.add_route('api.incoming.export', '/api/1.0/incoming/export')
    config.add_route('api.incoming.extensions', '/api/1.0/incoming/extensions')

    # Outgoing calls
    config.add_route('api.outgoing.answered', '/api/1.0/outgoing/answered')
    config.add_route('api.outgoing.answered-range', '/api/1.0/outgoing/answered/range')
    config.add_route('api.outgoing.answered-slice', '/api/1.0/outgoing/answered/slice')
    config.add_route('api.outgoing.answered-table', '/api/1.0/outgoing/answered/table')
    config.add_route('api.outgoing.duration', '/api/1.0/outgoing/duration/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.outgoing.duration-range', '/api/1.0/outgoing/duration/range')
    config.add_route('api.outgoing.duration-slice', '/api/1.0/outgoing/duration/slice/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.outgoing.missed', '/api/1.0/outgoing/missed')
    config.add_route('api.outgoing.missed-range', '/api/1.0/outgoing/missed/range')
    config.add_route('api.outgoing.missed-slice', '/api/1.0/outgoing/missed/slice')
    config.add_route('api.outgoing.missed-table', '/api/1.0/outgoing/missed/table')
    config.add_route('api.outgoing.ratio', '/api/1.0/outgoing/ratio')
    config.add_route('api.outgoing.ratio-range', '/api/1.0/outgoing/ratio/range')
    config.add_route('api.outgoing.ratio-slice', '/api/1.0/outgoing/ratio/slice')
    config.add_route('api.outgoing.stats', '/api/1.0/outgoing/stats')

    config.add_route('api.outgoing.export', '/api/1.0/outgoing/export')
    config.add_route('api.outgoing.extensions', '/api/1.0/outgoing/extensions')

    # Queue
    config.add_route('api.queue.abandoned', '/api/1.0/queue/abandoned')
    config.add_route('api.queue.abandoned-range', '/api/1.0/queue/abandoned/range')
    config.add_route('api.queue.abandoned-slice', '/api/1.0/queue/abandoned/slice')
    config.add_route('api.queue.abandoned-table', '/api/1.0/queue/abandoned/table')
    config.add_route('api.queue.answered', '/api/1.0/queue/answered')
    config.add_route('api.queue.answered-range', '/api/1.0/queue/answered/range')
    config.add_route('api.queue.answered-slice', '/api/1.0/queue/answered/slice')
    config.add_route('api.queue.answered-table', '/api/1.0/queue/answered/table')
    config.add_route('api.queue.duration', '/api/1.0/queue/duration/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.queue.duration-range', '/api/1.0/queue/duration/range')
    config.add_route('api.queue.duration-slice', '/api/1.0/queue/duration/slice/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.queue.holdtime', '/api/1.0/queue/holdtime/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.queue.holdtime-range', '/api/1.0/queue/holdtime/range/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.queue.holdtime-slice', '/api/1.0/queue/holdtime/slice/:aggregate', aggregate_predicate='aggregate')
    config.add_route('api.queue.missed', '/api/1.0/queue/missed')
    config.add_route('api.queue.missed-range', '/api/1.0/queue/missed/range')
    config.add_route('api.queue.missed-slice', '/api/1.0/queue/missed/slice')
    config.add_route('api.queue.missed-table', '/api/1.0/queue/missed/table')
    config.add_route('api.queue.ratio', '/api/1.0/queue/ratio')
    config.add_route('api.queue.ratio-range', '/api/1.0/queue/ratio/range')
    config.add_route('api.queue.ratio-slice', '/api/1.0/queue/ratio/slice')
    config.add_route('api.queue.stats', '/api/1.0/queue/stats')

    config.add_route('api.queue.export', '/api/1.0/queue/export')
    config.add_route('api.queue.extensions', '/api/1.0/queue/extensions')

    config.scan('ccmetrics.views')

    return config.make_wsgi_app()
