# -*- coding: utf-8 -*-

# CC-Metrics -- Call Center Metrics from Asterisk PBX
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2014 Enercoop, Easter-eggs
# https://gitlab.easter-eggs.com/ee/cc-metrics
#
# This file is part of CC-Metrics.
#
# CC-Metrics is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# CC-Metrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" PBX lookup for LDAP """


import ldap
import logging

from ccmetrics.extensions import ExtensionLookup
from rwlock import RWLock


log = logging.getLogger(__name__)


def make_connection(uri, bind_dn, bind_credential):
    # Trust self signed certificates
    ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)

    # Connect to LDAP server
    try:
        con = ldap.initialize(uri)
        con.protocol_version = ldap.VERSION3
        con.bind_s(bind_dn, bind_credential)
    except:
        log.exception('Failed to connect to LDAP server')
        con = None

    return con


class LDAPLookup(ExtensionLookup):

    def __init__(self, uri, bind_dn, bind_credential, basedn, filter, attrs):
        self.uri = uri
        self.bind_dn = bind_dn
        self.bind_credential = bind_credential
        self.basedn = basedn
        self.filter = filter
        self.attrs = [attr.strip() for attr in attrs.split()]

        self.rwlock = RWLock()

        # Calling inherited
        super(LDAPLookup, self).__init__()

    def __resolve(self, extension):
        # Connect to LDAP server
        con = make_connection(self.uri, self.bind_dn, self.bind_credential)
        if con is None:
            return extension

        # Search object associated with extension
        filter = self.filter.format(extension=extension)
        result = con.search_s(self.basedn, ldap.SCOPE_SUBTREE, filter, self.attrs)
        if result:
            # Get the first available value
            dn, attrs = result[0]
            for attr in self.attrs:
                if attr in attrs:
                    extension = attrs[attr][0].decode('utf-8')
                    break

        # Disconnect from LDAP server
        con.unbind()
        return extension

    def reset_cache(self):
        log.debug('Reloading extension lookup cache')
        self.rwlock.writer_acquire()

        self.cache = dict(dst={}, src={}, queue={})

        self.rwlock.writer_release()

    def resolve(self, kind, value):
        self.rwlock.reader_acquire()

        if value in self.cache[kind]:
            # Get resolved from cache
            resolved = self.cache[kind][value]
            self.rwlock.reader_release()
            return resolved

        self.rwlock.reader_release()

        # Store resolved to cache
        self.rwlock.writer_acquire()

        resolved = self.__resolve(value)
        self.cache[kind][value] = resolved

        self.rwlock.writer_release()

        return resolved
