# -*- coding: utf-8 -*-

# CC-Metrics -- Call Center Metrics from Asterisk PBX
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2014-2015 Enercoop, Easter-eggs
# https://gitlab.easter-eggs.com/ee/cc-metrics
#
# This file is part of CC-Metrics.
#
# CC-Metrics is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# CC-Metrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


""" CC Metrics common forms """

import datetime
import formencode
from formencode import validators
import logging

from pyramid_helpers.i18n import N_


log = logging.getLogger(__name__)


class DatetimeValidator(validators.RangeValidator):
    format = '%Y-%m-%d %H:%M:%S'

    messages = {
        'invalid': N_(u'Invalid value'),
    }

    def _to_python(self, value, state):
        try:
            return datetime.datetime.strptime(value, self.format)
        except:
            raise formencode.Invalid(self.message('invalid', state), value, state)

    def _from_python(self, value, state):
        try:
            return value.strftime(self.format)
        except:
            raise formencode.Invalid(self.message('invalid', state), value, state)


class SinceUntilValidator(validators.FormValidator):
    messages = {
        'missing': N_('Please enter a value'),
        'invalid': N_('Until date must be later than since date'),
    }

    strict = True

    def _to_python(self, field_dict, state):
        errors = {}

        if field_dict['since'] is not None and field_dict['until'] is not None:
            # Until date must be later than since date
            if (self.strict and field_dict['since'] >= field_dict['until']) or (not self.strict and field_dict['since'] > field_dict['until']):
                errors['since'] = formencode.Invalid(
                    self.message('invalid', state), field_dict['since'], state)
                errors['until'] = formencode.Invalid(
                    self.message('invalid', state), field_dict['until'], state)

        if errors:
            error_list = errors.items()
            error_list.sort()
            error_message = u', '.join([value.msg for name, value in error_list if value])
            raise formencode.Invalid(error_message, field_dict, state, error_dict=errors)

        # No error
        return field_dict

    def _from_python(self, value, state):
        return value
