# -*- coding: utf-8 -*-

# CC-Metrics -- Call Center Metrics from Asterisk PBX
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2014-2015 Enercoop, Easter-eggs
# https://gitlab.easter-eggs.com/ee/cc-metrics
#
# This file is part of CC-Metrics.
#
# CC-Metrics is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# CC-Metrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from pyramid.httpexceptions import HTTPFound
from pyramid.view import notfound_view_config
from pyramid.view import view_config

from pyramid_helpers.forms import validate
from pyramid_helpers.i18n import N_

from ccmetrics.forms import IncomingForm
from ccmetrics.forms import OutgoingForm
from ccmetrics.forms import RangeForm
from ccmetrics.forms import QueueForm
from ccmetrics.views.api.incoming import extensions as incoming_extensions
from ccmetrics.views.api.incoming import get_export as incoming_get_export
from ccmetrics.views.api.outgoing import extensions as outgoing_extensions
from ccmetrics.views.api.outgoing import get_export as outgoing_get_export
from ccmetrics.views.api.queue import extensions as queue_extensions
from ccmetrics.views.api.queue import get_export as queue_get_export


SUBTITLES = {
    'charts': N_('Charts'),
    'answered': N_('Answered'),
    'missed': N_('Missed'),
    'abandoned': N_('Abandoned'),
    'queue': N_('Queue'),
    'incoming': N_('Incoming'),
    'outgoing': N_('Outgoing'),
    'internal': N_('Internal'),
}


def manage_range(request):
    form = request.forms['filters']
    session = request.session

    if form.errors:
        return

    if form.result['since'] is None or form.result['until'] is None:
        # since and until are None when using persistent data from sesssion because they are configured as volatile items
        # Loading *shared* range values from session
        if 'range' in session:
            # Updating form with shared date range
            form.result.update(session['range'])
            form.from_python(form.result, validate=True)
    else:
        session['range'] = dict(since=form.result['since'], until=form.result['until'])


@notfound_view_config(renderer='/error-404.mako')
def error_404(request):
    _ = request.translate

    request.response.status_int = 404

    return dict(
        breadcrumb=[],
        subtitle=None,
        title=_(u'404 Error Page'),
    )


@view_config(route_name='index')
def index(request):
    # Reset session
    request.session.invalidate()

    # Go to default page
    registry = request.registry
    settings = registry.settings

    page = settings.get('dashboard.default', 'queue')
    if page not in ('incoming', 'internal', 'outgoing', 'queue'):
        page = 'queue'

    return HTTPFound(location=request.route_path('dashboard', page=page))


@view_config(route_name='dashboard', renderer='/dashboard.mako')
@validate('filters', RangeForm, method='get', persistent=True, volatile_items=('since', 'until', 'format'))
def dashboard(request):
    _ = request.translate
    page = request.matchdict['page']

    # Load/store range
    manage_range(request)

    breadcrumb = [
        (_(SUBTITLES[page]), request.route_path('dashboard', page=page)),
    ]

    title = _(SUBTITLES[page])
    subtitle = None

    return dict(
        breadcrumb=breadcrumb,
        page=page,
        subtitle=subtitle,
        title=title,
    )


@view_config(route_name='incoming', renderer='/incoming.mako')
@validate('filters', IncomingForm, method='get', persistent=True, volatile_items=('since', 'until', 'format'))
def incoming(request):
    _ = request.translate
    form = request.forms['filters']
    page = request.matchdict['page']

    # Load/store range
    manage_range(request)

    breadcrumb = [
        (_(u'Dashboard'), request.route_path('dashboard', page='incoming')),
        (_(u'Incoming'), request.route_path('incoming', page='charts')),
    ]
    if page != 'charts':
        breadcrumb.append((_(SUBTITLES[page]), request.route_path('incoming', page=page)))

    title = _(u'Incoming')
    subtitle = _(SUBTITLES[page])

    extensions = incoming_extensions(request)['extensions']
    extensions = [(k, v) for k, v in extensions.items()]
    extensions.sort(key=lambda extension: extension[1])

    if form.errors:
        return dict(
            breadcrumb=breadcrumb,
            extensions=extensions,
            page=page,
            subtitle=subtitle,
            title=title,
        )

    if form.result['format'] == 'csv':
        # CSV export
        since = form.result['since']
        until = form.result['until']

        dst = form.result['dst']
        internal = form.result['internal']

        request.override_renderer = 'csv'
        return incoming_get_export(request, since, until, dst=dst, internal=internal)

    return dict(
        breadcrumb=breadcrumb,
        extensions=extensions,
        page=page,
        subtitle=subtitle,
        title=title,
    )


@view_config(route_name='outgoing', renderer='/outgoing.mako')
@validate('filters', OutgoingForm, method='get', persistent=True, volatile_items=('since', 'until', 'format'))
def outgoing(request):
    _ = request.translate
    form = request.forms['filters']
    page = request.matchdict['page']

    # Load/store range
    manage_range(request)

    breadcrumb = [
        (_(u'Dashboard'), request.route_path('dashboard', page='outgoing')),
        (_(u'Outgoing'), request.route_path('outgoing', page='charts')),
    ]
    if page != 'charts':
        breadcrumb.append((_(SUBTITLES[page]), request.route_path('outgoing', page=page)))

    title = _(u'Outgoing')
    subtitle = _(SUBTITLES[page])

    extensions = outgoing_extensions(request)['extensions']
    extensions = [(k, v) for k, v in extensions.items()]
    extensions.sort(key=lambda extension: extension[1])

    if form.errors:
        return dict(
            breadcrumb=breadcrumb,
            extensions=extensions,
            page=page,
            subtitle=subtitle,
            title=title,
        )

    if form.result['format'] == 'csv':
        # CSV export
        since = form.result['since']
        until = form.result['until']

        internal = form.result['internal']
        src = form.result['src']

        request.override_renderer = 'csv'
        return outgoing_get_export(request, since, until, internal=internal, src=src)

    return dict(
        breadcrumb=breadcrumb,
        extensions=extensions,
        page=page,
        subtitle=subtitle,
        title=title,
    )


@view_config(route_name='queue', renderer='/queue.mako')
@validate('filters', QueueForm, method='get', persistent=True, volatile_items=('since', 'until', 'format'))
def queue(request):
    _ = request.translate
    form = request.forms['filters']
    page = request.matchdict['page']

    # Load/store range
    manage_range(request)

    breadcrumb = [
        (_(u'Dashboard'), request.route_path('dashboard', page='queue')),
        (_(u'Queue'), request.route_path('queue', page='charts')),
    ]
    if page != 'charts':
        breadcrumb.append((_(SUBTITLES[page]), request.route_path('queue', page=page)))

    title = _(u'Queue')
    subtitle = _(SUBTITLES[page])

    extensions = queue_extensions(request)['extensions']
    extensions = [(k, v) for k, v in extensions.items()]
    extensions.sort(key=lambda extension: extension[1])

    if form.errors:
        return dict(
            breadcrumb=breadcrumb,
            extensions=extensions,
            page=page,
            subtitle=subtitle,
            title=title,
        )

    if form.result['format'] == 'csv':
        # CSV export
        since = form.result['since']
        until = form.result['until']

        queue = form.result['queue']

        request.override_renderer = 'csv'
        return queue_get_export(request, since, until, queue=queue)

    return dict(
        breadcrumb=breadcrumb,
        extensions=extensions,
        page=page,
        subtitle=subtitle,
        title=title,
    )
