# -*- coding: utf-8 -*-

# CC-Metrics -- Call Center Metrics from Asterisk PBX
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2014-2015 Enercoop, Easter-eggs
# https://gitlab.easter-eggs.com/ee/cc-metrics
#
# This file is part of CC-Metrics.
#
# CC-Metrics is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# CC-Metrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sqlalchemy as sa

from pyramid.view import view_config
from pyramid_helpers.forms import validate

from ccmetrics.extensions import lookup as extensions_lookup
from ccmetrics.forms.api import OutgoingForm
from ccmetrics.models import CDR
from ccmetrics.models import DBSession

from common import get_aggregate
from common import get_step


def get_channel_clause(request, internal=False):
    """
    SQL clause for outgoing calls

    @param Request request, current Pyramid request object
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    registry = request.registry
    settings = registry.settings or {}

    trunks = [
        trunk.strip()
        for trunk in settings.get('asterisk.trunks', '').split()
    ]

    assert len(trunks) >= 1, 'Unconfigured asterisk.trunks variable'

    filters = []

    # Ensure not incoming
    filters.append(
        sa.and_(
            sa.not_(CDR.channel.startswith(trunk))
            for trunk in trunks
        )
    )

    if internal:
        # Ensure not outgoing
        filters.append(
            sa.and_(
                sa.not_(CDR.dstchannel.startswith(trunk))
                for trunk in trunks
            )
        )
    else:
        # Ensure outgoing
        filters.append(
            sa.or_(
                CDR.dstchannel.startswith(trunk)
                for trunk in trunks
            )
        )

    return sa.and_(*filters)


def get_answered(request, since, until, src=None, internal=False):
    """
    Get answered calls count by src

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Query
    query = DBSession.query(
        CDR.src,
        sa.func.count(CDR.uniqueid),
    ).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).group_by(CDR.src)

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = dict(
        (src, count)
        for src, count in query.all()
    )

    # Labels
    labels = dict(
        (src, extensions_lookup.resolve_src(src)) for src in result
    )

    return dict(
        count=len(result),
        labels=labels,
        result=result,
    )


def get_answered_range(request, since, until, src=None, internal=False):
    """
    Get answered calls count by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.

    !!!Warning: This function use PostgreSQL specific to_char function!!!
    """

    # Query
    query = DBSession.query(
        sa.cast(sa.func.to_char(CDR.calldate, 'HH24'), sa.Integer).label('hour'),
        sa.func.count(CDR.uniqueid),
    ).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).group_by('hour')

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for hour, count in query.all():
        result[hour] = count

    return dict(
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_answered_slice(request, since, until, src=None, internal=False):
    """
    Get answered calls count over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(CDR).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).order_by(sa.asc(CDR.calldate))

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = [0 for i in range(size)]
    index = 0
    for cdr in query.all():
        while cdr.calldate > steps[index] + step:
            index += 1

        result[index] += 1

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_answered_table(request, since, until, src=None, internal=False):
    """
    Get answered calls data for listing purpose

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    _ = request.translate

    # Query
    query = DBSession.query(CDR).filter(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    ).order_by(sa.asc(CDR.calldate))

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = [
        dict(
            calldate=cdr.calldate,
            clid=cdr.clid,
            src=extensions_lookup.resolve_src(cdr.src),
            dst=extensions_lookup.resolve_dst(cdr.dst) if internal else cdr.dst,
            duration=cdr.duration,
            billsec=cdr.billsec,
        )
        for cdr in query.all()
    ]

    # Headers
    headers = [
        dict(name='calldate', label=_(u'Call date'), type='date'),
        dict(name='clid', label=_(u'Caller Id')),
        dict(name='src', label=_(u'Caller')),
        dict(name='dst', label=_(u'Destination')),
        dict(name='duration', label=_(u'Duration'), type='duration'),
    ]
    if not internal:
        headers.append(
            dict(name='billsec', label=_(u'Billing count'), type='numeric')
        )

    return dict(
        count=len(result),
        headers=headers,
        result=result,
    )


def get_duration(request, since, until, src=None, internal=False):
    """
    Get answered calls duration by src

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Aggregate function
    aggregate_name, aggregate_func = get_aggregate(request)

    # Query
    query = DBSession.query(
        CDR.src,
        aggregate_func(sa.cast(CDR.duration, sa.Integer)),
    ).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).group_by(CDR.src)

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = dict(
        (src, value)
        for src, value in query.all()
    )

    # Labels
    labels = dict(
        (src, extensions_lookup.resolve_src(src)) for src in result
    )

    return dict(
        aggregate=aggregate_name,
        count=len(result),
        labels=labels,
        result=result,
    )


def get_duration_range(request, since, until, src=None, internal=False):
    """
    Get answered calls duration by range
    range is 0-5, 5-10, 10-15, 15+ (minutes)

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    _ = request.translate

    # Query
    query = DBSession.query(
        sa.case([
            (sa.between(sa.cast(CDR.duration, sa.Integer), 0, 300), 'r1'),
            (sa.between(sa.cast(CDR.duration, sa.Integer), 300, 600), 'r2'),
            (sa.between(sa.cast(CDR.duration, sa.Integer), 600, 900), 'r3'),
        ], else_='r4').label('range'),
        sa.func.count('range'),
    ).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).group_by('range')

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = dict(
        (range, count)
        for range, count in query.all()
    )

    # Labels
    labels = dict(
        r1=_('Less than 5 minutes'),
        r2=_('Between 5 and 10 minutes'),
        r3=_('Between 10 and 15 minutes'),
        r4=_('More than 15 minutes'),
    )

    return dict(
        count=len(result),
        labels=labels,
        result=result,
    )


def get_duration_slice(request, since, until, src=None, internal=False):
    """
    Get answered calls cumulative duration over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Aggregate function
    aggregate_name, aggregate_func = get_aggregate(request)

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(CDR).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).order_by(sa.asc(CDR.calldate))

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Collect raw data
    duration = [[] for i in range(size)]
    index = 0
    for cdr in query.all():
        while cdr.calldate > steps[index] + step:
            index += 1

        duration[index].append(int(cdr.duration))

    # Calculate metrics from raw data
    result = [0 for i in range(size)]
    for index in range(size):
        if duration[index]:
            if aggregate_name == 'avg':
                result[index] = sum(duration[index]) / float(len(duration[index]))
            elif aggregate_name == 'max':
                result[index] = max(duration[index])
            elif aggregate_name == 'min':
                result[index] = min(duration[index])
            elif aggregate_name == 'sum':
                result[index] = sum(duration[index])

    return dict(
        aggregate=aggregate_name,
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_export(request, since, until, src=None, internal=False):
    """
    CSV export

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    _ = request.translate

    # Query
    query = DBSession.query(CDR).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.lastapp == u'Dial',
    )).order_by(sa.asc(CDR.calldate))

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Values
    values = [
        dict(
            calldate=cdr.calldate,
            clid=cdr.clid,
            src=cdr.src,
            dst=cdr.dst,
            dcontext=cdr.dcontext,
            channel=cdr.channel,
            dstchannel=cdr.dstchannel,
            lastapp=cdr.lastapp,
            lastdata=cdr.lastdata,
            duration=cdr.duration,
            billsec=cdr.billsec,
            disposition=cdr.disposition,
            amaflags=cdr.amaflags,
            accountcode=cdr.accountcode,
            uniqueid=cdr.uniqueid,
        )
        for cdr in query.all()
    ]

    format = u'{calldate};"{clid}";"{src}";"{dst}";"{dcontext}";"{channel}";"{dstchannel}";"{lastapp}";"{lastdata}";{duration};{billsec};{disposition};{amaflags};{accountcode};{uniqueid}'
    header = u'calldate;clid;src;dst;dcontext;channel;dstchannel;lastapp;lastdata;duration;billsec;disposition;amaflags;accountcode;uniqueid'

    return dict(
        filename=_(u'outgoing.csv'),
        format=format,
        header=header,
        values=values,
    )


def get_missed(request, since, until, src=None, internal=False):
    """
    Get missed calls count by src

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Query
    query = DBSession.query(
        CDR.src,
        sa.func.count(CDR.uniqueid),
    ).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition != u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).group_by(CDR.src)

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = dict(
        (src, count)
        for src, count in query.all()
    )

    # Labels
    labels = dict(
        (src, extensions_lookup.resolve_src(src)) for src in result
    )

    return dict(
        count=len(result),
        labels=labels,
        result=result,
    )


def get_missed_range(request, since, until, src=None, internal=False):
    """
    Get missed calls count by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.

    !!!Warning: This function use PostgreSQL specific to_char function!!!
    """

    # Query
    query = DBSession.query(
        sa.cast(sa.func.to_char(CDR.calldate, 'HH24'), sa.Integer).label('hour'),
        sa.func.count(CDR.uniqueid),
    ).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition != u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).group_by('hour')

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for hour, count in query.all():
        result[hour] = count

    return dict(
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_missed_slice(request, since, until, src=None, internal=False):
    """
    Get missed calls count over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(CDR).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition != u'ANSWERED',
        CDR.lastapp == u'Dial',
    )).order_by(sa.asc(CDR.calldate))

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = [0 for i in range(size)]
    index = 0
    for cdr in query.all():
        while cdr.calldate > steps[index] + step:
            index += 1

        result[index] += 1

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_missed_table(request, since, until, src=None, internal=False):
    """
    Get missed calls data for listing purpose

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    _ = request.translate

    # Query
    query = DBSession.query(CDR).filter(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition != u'ANSWERED',
        CDR.lastapp == u'Dial',
    ).order_by(sa.asc(CDR.calldate))

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    # Result
    result = [
        dict(
            calldate=cdr.calldate,
            clid=cdr.clid,
            src=extensions_lookup.resolve_src(cdr.src),
            dst=extensions_lookup.resolve_dst(cdr.dst) if internal else cdr.dst,
            duration=cdr.duration,
            billsec=cdr.billsec,
            accountcode=cdr.accountcode,
        )
        for cdr in query.all()
    ]

    # Headers
    headers = [
        dict(name='calldate', label=_(u'Call date'), type='date'),
        dict(name='clid', label=_(u'Caller Id')),
        dict(name='src', label=_(u'Caller')),
        dict(name='dst', label=_(u'Destination')),
        dict(name='duration', label=_(u'Holdtime'), type='duration'),
    ]

    return dict(
        count=len(result),
        headers=headers,
        result=result,
    )


def get_ratio(request, since, until, src=None, internal=False):
    """
    Get missed calls ratio by src

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    answered = get_answered(request, since, until, src=src, internal=internal)['result']
    missed = get_missed(request, since, until, src=src, internal=internal)['result']

    result = dict()
    for src in set(answered.keys() + missed.keys()):
        total = answered.get(src, 0) + missed.get(src, 0)
        if total:
            result[src] = missed.get(src, 0) / float(total) * 100
        else:
            result[src] = 0

    return dict(
        count=len(result),
        result=result,
    )


def get_ratio_range(request, since, until, src=None, internal=False):
    """
    Get missed calls ratio by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    answered = get_answered_range(request, since, until, src=src, internal=internal)['result']
    missed = get_missed_range(request, since, until, src=src, internal=internal)['result']

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for index in range(size):
        total = answered[index] + missed[index]
        if total:
            result[index] = missed[index] / float(total) * 100

    return dict(
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_ratio_slice(request, since, until, src=None, internal=False):
    """
    Get missed calls ratio over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Steps
    steps, step, size = get_step(since, until)

    answered = get_answered_slice(request, since, until, src=src, internal=internal)['result']
    missed = get_missed_slice(request, since, until, src=src, internal=internal)['result']

    result = [0 for i in range(size)]
    for index in range(size):
        total = answered[index] + missed[index]
        if total:
            result[index] = missed[index] / float(total) * 100

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_stats(request, since, until, src=None, internal=False):
    """
    Get outgoing calls statistics

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string src, source to get data for, default None.
    @param boolean internal, whether to look for internal calls or not, default False.
    """

    # Answered
    # Query
    query = DBSession.query(
        sa.func.count(CDR.uniqueid),
    ).filter(sa.and_(
        CDR.calldate >= since,
        CDR.calldate < until,
        CDR.disposition == u'ANSWERED',
        CDR.lastapp == u'Dial',
    ))

    # Outgoing calls only
    channel_clause = get_channel_clause(request, internal=internal)
    query = query.filter(channel_clause)

    # Filters
    if src is not None:
        query = query.filter(CDR.src == src)

    answered = query.scalar()

    # Duration
    # Query
    query = query.with_entities(
        sa.func.avg(sa.cast(CDR.duration, sa.Integer)),
        sa.func.max(sa.cast(CDR.duration, sa.Integer)),
        sa.func.min(sa.cast(CDR.duration, sa.Integer)),
        sa.func.sum(sa.cast(CDR.duration, sa.Integer)),
    )

    result = query.first()
    avg_duration = int(result[0] or 0)
    max_duration = int(result[1] or 0)
    min_duration = int(result[2] or 0)
    sum_duration = int(result[3] or 0)

    return dict(
        answered=answered,
        # Duration
        avg_duration=avg_duration,
        max_duration=max_duration,
        min_duration=min_duration,
        sum_duration=sum_duration,
    )


ROUTES = {
    'api.outgoing.answered': get_answered,
    'api.outgoing.answered-range': get_answered_range,
    'api.outgoing.answered-slice': get_answered_slice,
    'api.outgoing.answered-table': get_answered_table,
    'api.outgoing.duration': get_duration,
    'api.outgoing.duration-range': get_duration_range,
    'api.outgoing.duration-slice': get_duration_slice,
    'api.outgoing.export': get_export,
    'api.outgoing.missed': get_missed,
    'api.outgoing.missed-range': get_missed_range,
    'api.outgoing.missed-slice': get_missed_slice,
    'api.outgoing.missed-table': get_missed_table,
    'api.outgoing.ratio': get_ratio,
    'api.outgoing.ratio-range': get_ratio_range,
    'api.outgoing.ratio-slice': get_ratio_slice,
    'api.outgoing.stats': get_stats,
}


@view_config(route_name='api.outgoing.answered', renderer='json')
@view_config(route_name='api.outgoing.answered-range', renderer='json')
@view_config(route_name='api.outgoing.answered-slice', renderer='json')
@view_config(route_name='api.outgoing.answered-table', renderer='json')
@view_config(route_name='api.outgoing.duration', renderer='json')
@view_config(route_name='api.outgoing.duration-range', renderer='json')
@view_config(route_name='api.outgoing.duration-slice', renderer='json')
@view_config(route_name='api.outgoing.export', renderer='csv')
@view_config(route_name='api.outgoing.missed', renderer='json')
@view_config(route_name='api.outgoing.missed-range', renderer='json')
@view_config(route_name='api.outgoing.missed-slice', renderer='json')
@view_config(route_name='api.outgoing.missed-table', renderer='json')
@view_config(route_name='api.outgoing.ratio', renderer='json')
@view_config(route_name='api.outgoing.ratio-range', renderer='json')
@view_config(route_name='api.outgoing.ratio-slice', renderer='json')
@view_config(route_name='api.outgoing.stats', renderer='json')
@validate('outgoing', OutgoingForm, method='get')
def outgoing(request):
    _ = request.translate
    form = request.forms['outgoing']

    # Prepare result
    result = dict(
        apiVersion='1.0',
        method=request.method,
        params=form.params,
        path=request.path,
        status=True,
        url=request.url,
    )

    if form.errors:
        result['status'] = False
        result['errors'] = form.errors
        result['messsage'] = _(u'Invalid or missing parameter')
        return result

    since = form.result['since']
    until = form.result['until']

    src = form.result['src']
    internal = form.result['internal']

    func = ROUTES.get(request.matched_route.name)
    if func is None:
        result['status'] = False
        result['messsage'] = _(u'Missing callable for route «{0}»').format(request.matched_route.name)
        return result

    result.update(func(request, since, until, src, internal))
    return result


@view_config(route_name='api.outgoing.extensions', renderer='json')
def extensions(request):
    # Prepare result
    result = dict(
        apiVersion='1.0',
        method=request.method,
        path=request.path,
        status=True,
        url=request.url,
    )

    # Query
    query = DBSession.query(
        sa.distinct(CDR.src),
    ).filter(CDR.lastapp == u'Dial')

    # Outgoing calls only
    channel_clause = get_channel_clause(request)
    query = query.filter(channel_clause)

    # Reset extensions' lookup cache
    extensions_lookup.reset_cache()

    result['extensions'] = dict(
        (src, extensions_lookup.resolve_src(src)) for src, in query.all()
    )

    return result
