# -*- coding: utf-8 -*-

# CC-Metrics -- Call Center Metrics from Asterisk PBX
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2014-2015 Enercoop, Easter-eggs
# https://gitlab.easter-eggs.com/ee/cc-metrics
#
# This file is part of CC-Metrics.
#
# CC-Metrics is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# CC-Metrics is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import re
import sqlalchemy as sa

from pyramid.view import view_config
from pyramid_helpers.forms import validate

from ccmetrics.extensions import lookup as extensions_lookup
from ccmetrics.forms.api import QueueForm
from ccmetrics.models import DBSession
from ccmetrics.models import QueueLog

from common import get_aggregate
from common import get_step


LOCAL_CHANNEL_RE = re.compile('Local/(?P<extension>.*)@(?P<context>.*)')


def get_abandoned(request, since, until, queue=None):
    """
    Get abandoned calls count by queue

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Query
    query = DBSession.query(
        QueueLog.queuename,
        sa.func.count(QueueLog.id),
    ).filter(
        QueueLog.event.in_((u'EXITWITHKEY', u'ABANDON')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).group_by(QueueLog.queuename)

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = dict(
        (queuename, count)
        for queuename, count in query.all()
    )

    # Labels
    labels = dict(
        (queuename, extensions_lookup.resolve_queue(queuename)) for queuename in result
    )

    return dict(
        count=len(result),
        labels=labels,
        result=result,
    )


def get_abandoned_range(request, since, until, queue=None):
    """
    Get abandoned calls count by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.

    !!!Warning: This function use PostgreSQL specific to_char function!!!
    """

    # Query
    query = DBSession.query(
        sa.cast(sa.func.to_char(QueueLog.time, 'HH24'), sa.Integer).label('hour'),
        sa.func.count(QueueLog.id),
    ).filter(
        QueueLog.event.in_((u'EXITWITHKEY', u'ABANDON')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).group_by('hour')

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for hour, count in query.all():
        result[hour] = count

    return dict(
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_abandoned_slice(request, since, until, queue=None):
    """
    Get abandoned calls count over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(QueueLog).filter(sa.and_(
        QueueLog.event.in_((u'EXITWITHKEY', u'ABANDON')),
        QueueLog.time >= since,
        QueueLog.time < until,
    )).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = [0 for i in range(size)]
    index = 0
    for queue_log in query.all():
        while queue_log.time > steps[index] + step:
            index += 1

        result[index] += 1

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_abandoned_table(request, since, until, queue=None):
    """
    Get abandoned calls data for listing purpose

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    _ = request.translate

    # Query
    query = DBSession.query(QueueLog).filter(
        QueueLog.event.in_((u'EXITWITHKEY', u'ABANDON')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = [
        dict(
            callerid=queue_log.callerid,
            queuename=extensions_lookup.resolve_queue(queue_log.queuename),
            waittime=int(queue_log.data3) if queue_log.event == u'ABANDON' else None,
            position=int(queue_log.data1) if queue_log.event == u'ABANDON' else int(queue_log.data2),
            time=queue_log.time,
        )
        for queue_log in query.all()
    ]

    # Headers
    headers = [
        dict(name='time', label=_(u'Call date'), type='date'),
        dict(name='queuename', label=_(u'Queue')),
        dict(name='callerid', label=_(u'Caller')),
        dict(name='waittime', label=_(u'Wait time'), type='duration'),
        dict(name='position', label=_(u'Position'), type='integer'),
    ]

    return dict(
        count=len(result),
        headers=headers,
        result=result,
    )


def get_answered(request, since, until, queue=None):
    """
    Get answered calls count by queue or by agent if queue param is provided

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    if queue is None:
        # No queue, grouping by queue
        # Query
        query = DBSession.query(
            QueueLog.queuename,
            sa.func.count(QueueLog.id),
        ).filter(sa.and_(
            QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER', u'TRANSFER')),
            QueueLog.time >= since,
            QueueLog.time < until,
        )).group_by(QueueLog.queuename)

        # Result
        result = dict(
            (queue, count)
            for queue, count in query.all()
        )

        # Labels
        labels = dict(
            (queue, extensions_lookup.resolve_queue(queue)) for queue in result
        )

    else:
        # Queue is provided, grouping by agent
        # Query
        query = DBSession.query(
            QueueLog.agent,
            sa.func.count(QueueLog.id),
        ).filter(sa.and_(
            QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER', u'TRANSFER')),
            QueueLog.time >= since,
            QueueLog.time < until,
            QueueLog.queuename == queue,
        )).group_by(QueueLog.agent)

        # Result - extracting extension from agent
        result = dict(
            (LOCAL_CHANNEL_RE.sub(lambda m: m.group('extension'), dst), count)
            for dst, count in query.all()
        )

        # Labels
        labels = dict(
            (dst, extensions_lookup.resolve_dst(dst)) for dst in result
        )

    return dict(
        count=len(result),
        labels=labels,
        result=result,
    )


def get_answered_range(request, since, until, queue=None):
    """
    Get answered calls count by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.

    !!!Warning: This function use PostgreSQL specific to_char function!!!
    """

    # Query
    query = DBSession.query(
        sa.cast(sa.func.to_char(QueueLog.time, 'HH24'), sa.Integer).label('hour'),
        sa.func.count(QueueLog.id),
    ).filter(
        QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER', u'TRANSFER')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).group_by('hour')

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for hour, count in query.all():
        result[hour] = count

    return dict(
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_answered_slice(request, since, until, queue=None):
    """
    Get answered calls count over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(QueueLog).filter(sa.and_(
        QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER', u'TRANSFER')),
        QueueLog.time >= since,
        QueueLog.time < until,
    )).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Results
    result = [0 for i in range(size)]
    index = 0
    for queue_log in query.all():
        while queue_log.time > steps[index] + step:
            index += 1

        result[index] += 1

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_answered_table(request, since, until, queue=None):
    """
    Get answered calls data for listing purpose

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    _ = request.translate

    # Query
    query = DBSession.query(QueueLog).filter(
        QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER', u'TRANSFER')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = [
        dict(
            agent=extensions_lookup.resolve_dst(LOCAL_CHANNEL_RE.sub(lambda m: m.group('extension'), queue_log.agent)),
            callerid=queue_log.callerid,
            calltime=int(queue_log.data4) if queue_log.event == u'TRANSFER' else int(queue_log.data2),
            holdtime=int(queue_log.data3) if queue_log.event == u'TRANSFER' else int(queue_log.data1),
            queuename=extensions_lookup.resolve_queue(queue_log.queuename),
            time=queue_log.time,
        )
        for queue_log in query.all()
    ]

    # Headers
    headers = [
        dict(name='time', label=_(u'Call date'), type='date'),
        dict(name='queuename', label=_(u'Queue')),
        dict(name='callerid', label=_(u'Caller')),
        dict(name='agent', label=_(u'Agent')),
        dict(name='holdtime', label=_(u'Holdtime'), type='duration'),
        dict(name='calltime', label=_(u'Duration'), type='duration'),
    ]

    return dict(
        count=len(result),
        headers=headers,
        result=result,
    )


def get_duration(request, since, until, queue=None):
    """
    Get answered calls duration by queue or by agent if queue param is provided

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Aggregate function
    aggregate_name, aggregate_func = get_aggregate(request)

    if queue is None:
        # No queue, grouping by queue
        # Query
        query = DBSession.query(
            QueueLog.queuename,
            aggregate_func(sa.cast(QueueLog.data2, sa.Integer)),
        ).filter(sa.and_(
            QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER')),
            QueueLog.time >= since,
            QueueLog.time < until,
        )).group_by(QueueLog.queuename)

        # Result
        result = dict(
            (queuename, value)
            for queuename, value in query.all()
        )

        # Labels
        labels = dict(
            (queuename, extensions_lookup.resolve_queue(queuename)) for queuename in result
        )

    else:
        # Queue is provided, grouping by agent
        # Query
        query = DBSession.query(
            QueueLog.agent,
            aggregate_func(sa.cast(QueueLog.data2, sa.Integer)),
        ).filter(sa.and_(
            QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER')),
            QueueLog.time >= since,
            QueueLog.time < until,
            QueueLog.queuename == queue,
        )).group_by(QueueLog.agent)

        # Result - extracting extension from agent
        result = dict(
            (LOCAL_CHANNEL_RE.sub(lambda m: m.group('extension'), dst), value)
            for dst, value in query.all()
        )

        # Labels
        labels = dict(
            (dst, extensions_lookup.resolve_dst(dst)) for dst in result
        )

    return dict(
        aggregate=aggregate_name,
        count=len(result),
        labels=labels,
        result=result,
    )


def get_duration_range(request, since, until, queue=None):
    """
    Get answered calls duration by range
    range is 0-5, 5-10, 10-15, 15+ (minutes)

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    _ = request.translate

    # Query
    query = DBSession.query(
        sa.case([
            (sa.between(sa.cast(QueueLog.data2, sa.Integer), 0, 300), 'r1'),
            (sa.between(sa.cast(QueueLog.data2, sa.Integer), 300, 600), 'r2'),
            (sa.between(sa.cast(QueueLog.data2, sa.Integer), 600, 900), 'r3'),
        ], else_='r4').label('range'),
        sa.func.count('range'),
    ).filter(sa.and_(
        QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER')),
        QueueLog.time >= since,
        QueueLog.time < until,
    )).group_by('range')

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = dict(
        (range, count)
        for range, count in query.all()
    )

    # Labels
    labels = dict(
        r1=_('Less than 5 minutes'),
        r2=_('Between 5 and 10 minutes'),
        r3=_('Between 10 and 15 minutes'),
        r4=_('More than 15 minutes'),
    )

    return dict(
        count=len(result),
        labels=labels,
        result=result,
    )


def get_duration_slice(request, since, until, queue=None):
    """
    Get answered calls cumulative duration over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Aggregate function
    aggregate_name, aggregate_func = get_aggregate(request)

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(QueueLog).filter(sa.and_(
        QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER')),
        QueueLog.time >= since,
        QueueLog.time < until,
    )).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Collect raw data
    duration = [[] for i in range(size)]
    index = 0
    for queue_log in query.all():
        while queue_log.time > steps[index] + step:
            index += 1

        duration[index].append(int(queue_log.data2))

    # Calculate metrics from raw data
    result = [0 for i in range(size)]
    for index in range(size):
        if duration[index]:
            if aggregate_name == 'avg':
                result[index] = sum(duration[index]) / float(len(duration[index]))
            elif aggregate_name == 'max':
                result[index] = max(duration[index])
            elif aggregate_name == 'min':
                result[index] = min(duration[index])
            elif aggregate_name == 'sum':
                result[index] = sum(duration[index])

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_holdtime(request, since, until, queue=None):
    """
    Get (max, min, avg) holdtime by queue

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Aggregate function
    aggregate_name, aggregate_func = get_aggregate(request)

    # Query
    query = DBSession.query(
        QueueLog.queuename,
        aggregate_func(sa.cast(QueueLog.data1, sa.Integer)),
    ).filter(sa.and_(
        QueueLog.event == u'CONNECT',
        QueueLog.time >= since,
        QueueLog.time < until,
    )).group_by(QueueLog.queuename)

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = dict(
        (queuename, value)
        for queuename, value in query.all()
    )

    # Labels
    labels = dict(
        (queuename, extensions_lookup.resolve_queue(queuename)) for queuename in result
    )

    return dict(
        aggregate=aggregate_name,
        count=len(result),
        labels=labels,
        result=result,
    )


def get_holdtime_range(request, since, until, queue=None):
    """
    Get (max, min, avg) holdtime by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Aggregate function
    aggregate_name, aggregate_func = get_aggregate(request)

    # Query
    query = DBSession.query(
        sa.cast(sa.func.to_char(QueueLog.time, 'HH24'), sa.Integer).label('hour'),
        aggregate_func(sa.cast(QueueLog.data1, sa.Integer)),
    ).filter(sa.and_(
        QueueLog.event == u'CONNECT',
        QueueLog.time >= since,
        QueueLog.time < until,
    )).group_by('hour')

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for hour, value in query.all():
        result[hour] = int(value)

    return dict(
        aggregate=aggregate_name,
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_holdtime_slice(request, since, until, queue=None):
    """
    Get (max, min, avg) holdtime by queue over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Aggregate function
    aggregate_name, aggregate_func = get_aggregate(request)

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(QueueLog).filter(sa.and_(
        QueueLog.event == u'CONNECT',
        QueueLog.time >= since,
        QueueLog.time < until,
    )).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Collect raw data
    holdtime = [[] for i in range(size)]
    index = 0
    for queue_log in query.all():
        while queue_log.time > steps[index] + step:
            index += 1

        holdtime[index].append(int(queue_log.data1))

    # Calculate metrics from raw data
    result = [0 for i in range(size)]
    for index in range(size):
        if holdtime[index]:
            if aggregate_name == 'avg':
                result[index] = sum(holdtime[index]) / float(len(holdtime[index]))
            elif aggregate_name == 'max':
                result[index] = max(holdtime[index])
            elif aggregate_name == 'min':
                result[index] = min(holdtime[index])
            elif aggregate_name == 'sum':
                result[index] = sum(holdtime[index])

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_export(request, since, until, queue=None):
    """
    CSV export

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    _ = request.translate

    # Query
    query = DBSession.query(QueueLog).filter(sa.and_(
        QueueLog.event.in_((
            u'ABANDON',             # The caller abandoned their position in the queue
            u'COMPLETEAGENT',       # The caller was connected to an agent, and the call was terminated normally by the *agent*
            u'COMPLETECALLER',      # The caller was connected to an agent, and the call was terminated normally by the *caller*
            u'CONNECT',             # The caller was connected to an agent
            u'EXITEMPTY',           # The caller was exited from the queue forcefully because the queue had no reachable members
            u'EXITWITHKEY',         # The caller elected to use a menu key to exit the queue
            u'EXITWITHTIMEOUT'      # The caller was on hold too long and the timeout expired
            u'TRANSFER',            # Caller was transferred to a different extension
        )),
        QueueLog.time >= since,
        QueueLog.time < until,
    )).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Values
    values = [
        dict(
            time=queue_log.time,
            callid=queue_log.callid,
            queuename=queue_log.queuename,
            agent=queue_log.agent,
            event=queue_log.event,
            data1=queue_log.data1,
            data2=queue_log.data2,
            data3=queue_log.data3,
            data4=queue_log.data4,
            data5=queue_log.data5,
        )
        for queue_log in query.all()
    ]

    format = u'{time};{callid};{queuename};{agent};{event};{data1};{data2};{data3};{data4};{data5}'
    header = u'time;callid;queuename;agent;event;data1;data2;data3;data4;data5'
    return dict(
        filename=_(u'queue.csv'),
        format=format,
        header=header,
        values=values,
    )


def get_missed(request, since, until, queue=None):
    """
    Get missed calls count by queue

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Query
    query = DBSession.query(
        QueueLog.queuename,
        sa.func.count(QueueLog.id),
    ).filter(
        QueueLog.event.in_((u'EXITEMPTY', u'EXITWITHTIMEOUT')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).group_by(QueueLog.queuename)

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = dict(
        (queuename, count)
        for queuename, count in query.all()
    )

    # Labels
    labels = dict(
        (queuename, extensions_lookup.resolve_queue(queuename)) for queuename in result
    )

    return dict(
        count=len(result),
        labels=labels,
        result=result,
    )


def get_missed_range(request, since, until, queue=None):
    """
    Get missed calls count by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.

    !!!Warning: This function use PostgreSQL specific to_char function!!!
    """

    # Query
    query = DBSession.query(
        sa.cast(sa.func.to_char(QueueLog.time, 'HH24'), sa.Integer).label('hour'),
        sa.func.count(QueueLog.id),
    ).filter(
        QueueLog.event.in_((u'EXITEMPTY', u'EXITWITHTIMEOUT')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).group_by('hour')

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for hour, count in query.all():
        result[hour] = count

    return dict(
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_missed_slice(request, since, until, queue=None):
    """
    Get missed calls count over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Steps
    steps, step, size = get_step(since, until)

    # Query
    query = DBSession.query(QueueLog).filter(sa.and_(
        QueueLog.event.in_((u'EXITEMPTY', u'EXITWITHTIMEOUT')),
        QueueLog.time >= since,
        QueueLog.time < until,
    )).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = [0 for i in range(size)]
    index = 0
    for queue_log in query.all():
        while queue_log.time > steps[index] + step:
            index += 1

        result[index] += 1

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_missed_table(request, since, until, queue=None):
    """
    Get missed calls data for listing purpose

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    _ = request.translate

    # Query
    query = DBSession.query(QueueLog).filter(
        QueueLog.event.in_((u'EXITEMPTY', u'EXITWITHTIMEOUT')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ).order_by(sa.asc(QueueLog.time))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    # Result
    result = [
        dict(
            callerid=queue_log.callerid,
            position=int(queue_log.data1),
            queuename=extensions_lookup.resolve_queue(queue_log.queuename),
            time=queue_log.time,
        )
        for queue_log in query.all()
    ]

    # Headers
    headers = [
        dict(name='time', label=_(u'Call date'), type='date'),
        dict(name='queuename', label=_(u'Queue')),
        dict(name='callerid', label=_(u'Caller')),
        dict(name='position', label=_(u'Position'), type='integer'),
    ]

    return dict(
        count=len(result),
        headers=headers,
        result=result,
    )


def get_ratio(request, since, until, queue=None):
    """
    Get missed calls ratio by queue

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    answered = get_answered(request, since, until, queue=queue)['result']
    missed = get_missed(request, since, until, queue=queue)['result']

    result = dict()
    for queue in set(answered.keys() + missed.keys()):
        total = answered.get(queue, 0) + missed.get(queue, 0)
        if total:
            result[queue] = missed.get(queue, 0) / float(total) * 100
        else:
            result[queue] = 0

    return dict(
        count=len(result),
        result=result,
    )


def get_ratio_range(request, since, until, queue=None):
    """
    Get missed calls ratio by hour

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    answered = get_answered_range(request, since, until, queue=queue)['result']
    missed = get_missed_range(request, since, until, queue=queue)['result']

    # Result
    size = 24
    steps = ['{0}h'.format(hour) for hour in range(size)]
    result = [0 for i in range(size)]
    for index in range(size):
        total = answered[index] + missed[index]
        if total:
            result[index] = missed[index] / float(total) * 100

    return dict(
        count=size,
        result=result,
        step=dict(hours=1),
        steps=steps,
    )


def get_ratio_slice(request, since, until, queue=None):
    """
    Get missed calls ratio over time by step
    Step value is calculated to be an «appropriate» value for [since, until] range

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Steps
    steps, step, size = get_step(since, until)

    answered = get_answered_slice(request, since, until, queue=queue)['result']
    missed = get_missed_slice(request, since, until, queue=queue)['result']

    result = [0 for i in range(size)]
    for index in range(size):
        total = answered[index] + missed[index]
        if total:
            result[index] = missed[index] / float(total) * 100

    return dict(
        count=size,
        result=result,
        step=dict(months=step.months, days=step.days, hours=step.hours),
        steps=steps,
    )


def get_stats(request, since, until, queue=None):
    """
    Get queue calls statistics

    @param Request request, current Pyramid request object
    @param datetime since, since date to collect data from
    @param datetime until, until date to collect data to
    @param string queue, queue to get data for, default None.
    """

    # Answered
    # Query
    query = DBSession.query(
        sa.func.count(QueueLog.id),
    ).filter(sa.and_(
        QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER', u'TRANSFER')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ))

    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    answered = query.scalar()

    # Duration
    # Query
    query = DBSession.query(
        sa.func.avg(sa.cast(QueueLog.data2, sa.Integer)),
        sa.func.max(sa.cast(QueueLog.data2, sa.Integer)),
        sa.func.min(sa.cast(QueueLog.data2, sa.Integer)),
        sa.func.sum(sa.cast(QueueLog.data2, sa.Integer)),
    ).filter(sa.and_(
        QueueLog.event.in_((u'COMPLETEAGENT', u'COMPLETECALLER')),
        QueueLog.time >= since,
        QueueLog.time < until,
    ))

    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    result = query.first()
    avg_duration = int(result[0] or 0)
    max_duration = int(result[1] or 0)
    min_duration = int(result[2] or 0)
    sum_duration = int(result[3] or 0)

    # Holdtime
    # Query
    query = DBSession.query(
        sa.func.avg(sa.cast(QueueLog.data1, sa.Integer)),
        sa.func.max(sa.cast(QueueLog.data1, sa.Integer)),
        sa.func.min(sa.cast(QueueLog.data1, sa.Integer)),
    ).filter(sa.and_(
        QueueLog.event == u'CONNECT',
        QueueLog.time >= since,
        QueueLog.time < until,
    ))

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    result = query.first()
    avg_holdtime = int(result[0] or 0)
    max_holdtime = int(result[1] or 0)
    min_holdtime = int(result[2] or 0)

    # Missed
    # Query
    query = DBSession.query(
        sa.func.count(QueueLog.id),
    ).filter(
        QueueLog.event.in_((u'EXITEMPTY', u'EXITWITHTIMEOUT')),
        QueueLog.time >= since,
        QueueLog.time < until,
    )

    # Filters
    if queue is not None:
        query = query.filter(QueueLog.queuename == queue)

    missed = query.scalar()

    # Ratios
    total = answered + missed
    if total:
        answered_ratio = round(answered / float(total) * 100, 2)
        missed_ratio = round(missed / float(total) * 100, 2)
    else:
        answered_ratio = 0
        missed_ratio = 0

    return dict(
        answered=answered,
        answered_ratio=answered_ratio,
        # Duration
        avg_duration=avg_duration,
        max_duration=max_duration,
        min_duration=min_duration,
        sum_duration=sum_duration,
        # Holdtime
        avg_holdtime=avg_holdtime,
        max_holdtime=max_holdtime,
        min_holdtime=min_holdtime,
        missed=missed,
        missed_ratio=missed_ratio,
        total=total,
    )


ROUTES = {
    'api.queue.abandoned': get_abandoned,
    'api.queue.abandoned-range': get_abandoned_range,
    'api.queue.abandoned-slice': get_abandoned_slice,
    'api.queue.abandoned-table': get_abandoned_table,
    'api.queue.answered': get_answered,
    'api.queue.answered-range': get_answered_range,
    'api.queue.answered-slice': get_answered_slice,
    'api.queue.answered-table': get_answered_table,
    'api.queue.duration': get_duration,
    'api.queue.duration-range': get_duration_range,
    'api.queue.duration-slice': get_duration_slice,
    'api.queue.export': get_export,
    'api.queue.holdtime': get_holdtime,
    'api.queue.holdtime-range': get_holdtime_range,
    'api.queue.holdtime-slice': get_holdtime_slice,
    'api.queue.missed': get_missed,
    'api.queue.missed-range': get_missed_range,
    'api.queue.missed-slice': get_missed_slice,
    'api.queue.missed-table': get_missed_table,
    'api.queue.ratio': get_ratio,
    'api.queue.ratio-slice': get_ratio_slice,
    'api.queue.ratio-range': get_ratio_range,
    'api.queue.stats': get_stats,
}


@view_config(route_name='api.queue.abandoned', renderer='json')
@view_config(route_name='api.queue.abandoned-range', renderer='json')
@view_config(route_name='api.queue.abandoned-slice', renderer='json')
@view_config(route_name='api.queue.abandoned-table', renderer='json')
@view_config(route_name='api.queue.answered', renderer='json')
@view_config(route_name='api.queue.answered-range', renderer='json')
@view_config(route_name='api.queue.answered-slice', renderer='json')
@view_config(route_name='api.queue.answered-table', renderer='json')
@view_config(route_name='api.queue.duration', renderer='json')
@view_config(route_name='api.queue.duration-range', renderer='json')
@view_config(route_name='api.queue.duration-slice', renderer='json')
@view_config(route_name='api.queue.export', renderer='csv')
@view_config(route_name='api.queue.holdtime', renderer='json')
@view_config(route_name='api.queue.holdtime-range', renderer='json')
@view_config(route_name='api.queue.holdtime-slice', renderer='json')
@view_config(route_name='api.queue.missed', renderer='json')
@view_config(route_name='api.queue.missed-range', renderer='json')
@view_config(route_name='api.queue.missed-slice', renderer='json')
@view_config(route_name='api.queue.missed-table', renderer='json')
@view_config(route_name='api.queue.ratio', renderer='json')
@view_config(route_name='api.queue.ratio-range', renderer='json')
@view_config(route_name='api.queue.ratio-slice', renderer='json')
@view_config(route_name='api.queue.stats', renderer='json')
@validate('queue', QueueForm, method='get')
def queue(request):
    _ = request.translate
    form = request.forms['queue']

    # Prepare result
    result = dict(
        apiVersion='1.0',
        method=request.method,
        params=form.params,
        path=request.path,
        status=True,
        url=request.url,
    )

    if form.errors:
        result['status'] = False
        result['errors'] = form.errors
        result['messsage'] = _(u'Invalid or missing parameter')
        return result

    since = form.result['since']
    until = form.result['until']

    queue = form.result['queue']

    func = ROUTES.get(request.matched_route.name)
    if func is None:
        result['status'] = False
        result['messsage'] = _(u'Missing callable for route «{0}»').format(request.matched_route.name)
        return result

    result.update(func(request, since, until, queue))
    return result


@view_config(route_name='api.queue.extensions', renderer='json')
def extensions(request):
    # Prepare result
    result = dict(
        apiVersion='1.0',
        method=request.method,
        path=request.path,
        status=True,
        url=request.url,
    )

    # Query
    query = DBSession.query(
        sa.distinct(QueueLog.queuename)
    ).filter(QueueLog.event == u'ENTERQUEUE')

    # Reset extensions' lookup cache
    extensions_lookup.reset_cache()

    result['extensions'] = dict(
        (queue, extensions_lookup.resolve_queue(queue)) for queue, in query.all()
    )

    return result
