/*
 * CC-Metrics -- Call Center Metrics from Asterisk PBX
 * By: Cyril Lacoux <clacoux@easter-eggs.com>
 *
 * Copyright (C) 2014-2015 Enercoop, Easter-eggs
 * https://gitlab.easter-eggs.com/ee/cc-metrics
 *
 * This file is part of CC-Metrics.
 *
 * CC-Metrics is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * CC-Metrics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Global variables
 */

// Chart variables
var colors = [
    // [normal, highlight]
    ['#F56954', '#D33724'],  // red
    ['#00A65A', '#008D4C'],  // green
    ['#F39C12', '#DB8B0B'],  // yellow
    ['#00C0EF', '#00A7D0'],  // light-blue
    ['#3C8DBC', '#357CA5'],  // blue
    ['#D2D6DE', '#B5BBC8'],  // grey
    ['#596D82', '#001A35'],  // navy
    ['#605CA8', '#001A35'],  // purple
    ['#39CCCC', '#30BBBB'],  // teal
    ['#D81B60', '#CA195A']   // maroon
];

var chartOptions = {};

chartOptions.lines = {
    animation: false,
    responsive: true,
    maintainAspectRatio: false,
    barStrokeWidth: 1,
    barValueSpacing: 1,
    tooltipFillColor: '#222D32',
    tooltipFontFamily: 'Source Sans Pro'
};

chartOptions.pie = {
    animation: false,
    responsive: true,
    maintainAspectRatio: false,
    legendTemplate: '<ul class="list-unstyled <%=name.toLowerCase()%>-legend"><% for (var i=0; i<segments.length; i++){%><li><i class="fa fa-square" style="color:<%=segments[i].fillColor%>"></i>&nbsp;<%if(segments[i].label){%><%=segments[i].label%><%}%></li><%}%></ul>',
    tooltipFillColor: '#222D32',
    tooltipFontFamily: 'Source Sans Pro'
};

// Custom date ranges for daterangepicker
var ranges = {};


/*
 * I18n
 */
function _(string) {
    return translations.gettext(string);
}


/*
 * Value converters
 */
var converters = {
    hours: function(value) { return value / 3600; },
    minutes: function(value) { return value / 60; },
    value: function(value) { return value; }
};


/*
 * Value formatters
 */
var formatters = {
    durationHours: function(value) {
        return formatters.duration(value * 3600);
    },
    durationMinutes: function(value) {
        return formatters.duration(value * 60);
    },
    duration: function(value) {
        var h, m;
        var s = parseInt(value, 10);
        var hr = Array();

        // Hours
        h = Math.floor(s / 3600);
        if (h) {
            hr.push(sprintf('%dh', h));
            s = s % 3600;
        }

        // Minutes
        m = Math.floor(s / 60);
        if (m) {
            hr.push(sprintf(h ? '%02dm' : '%dm', m));
            s = s % 60;
        }

        if (s) {
            hr.push(sprintf((h || m) ? '%02ds' : '%ds', s));
        }

        return hr.join('');
    },
    percent: function(value) {
        return sprintf('%.1f%%', value);
    },
    integer: function(value) {
        return sprintf('%d', Math.round(value));
    },
    value: function(value) {
        return sprintf('%.1f', value);
    }
};


function getTopTen(array) {
    // Sort and slice
    array.sort(function(a, b) { return b.value - a.value });
    array = array.slice(0, 10);

    // Add colors
    for (var i=0; i<array.length; i++) {
        array[i]['color'] = colors[i][0];
        array[i]['highlight'] = colors[i][1];
    }

    return array;
}


function displayError(placeholder, message) {
    // Display error message
    placeholder.html('<div class="alert alert-danger"><h4><i class="icon fa fa-ban"> </i>' + _('Error!') + '</h4>' + message + '</div>');
}


function doBarChart(placeholder, data, total, formatter) {
    var options = {};
    options = $.extend(options, chartOptions.lines);

    if (!formatter || !(formatter in formatters)) {
        formatter = 'value';
    }

    // Set templates
    options.tooltipTemplate = '<%if (label){%><%=label%> : <%}%><%=formatters.' + formatter + '(value)%>';

    // Get context
    var canvas = $('canvas', placeholder).get(0).getContext('2d');
    var chart = new Chart(canvas).Bar(data, options);

    $(placeholder).data('chart', chart);
}


function doPieChart(placeholder, dataset, total, formatter) {
    var options = {};
    $.extend(options, chartOptions.pie);

    if (!formatter || !(formatter in formatters)) {
        formatter = 'percent';
    }

    if (formatter == 'percent') {
        for (var i=0; i<dataset.length; i++) {
            dataset[i].value = dataset[i].value * 100 / total;
        }
    }

    // Get only top ten
    dataset = getTopTen(dataset);

    // Set templates
    options.tooltipTemplate = '<%if (label){%><%=label%> : <%}%><%=formatters.' + formatter + '(value)%>';

    // Get context
    var canvas = $('canvas', placeholder).get(0).getContext('2d');
    var chart = new Chart(canvas).Doughnut(dataset, options);

    // Add legend
    var legend = chart.generateLegend();
    $('.chart-legend', placeholder).append(legend);

    $(placeholder).data('chart', chart);
}


function initCharts(wsParams) {
    var data, dataset, series, url;

    wsParams.since = $('#filters-since').val();
    wsParams.until = moment($('#filters-until').val()).add(1, 'day').toISOString();

    // Pies
    $('.chart-metrics.chart-pie').each(function(index, element) {
        var placeholder = $(element);
        var url = placeholder.data('url');
        var formatter = placeholder.data('formatter') || 'value';
        var totalFormatter = placeholder.data('total') ? formatters[placeholder.data('total')] : null;
        var total = 0;

        placeholder.parents('.box:first').append('<div class="overlay"><i class="fa fa-circle-o-notch fa-spin fa-4x"> </i></div>');

        $.getJSON(url, wsParams, function(data) {
            if (data && data.status === true) {
                if (data.count > 0) {
                    // Construct dataset
                    dataset = [];
                    for (var label in data.result) {
                        dataset.push({
                            label: data.labels[label] || label,
                            value: data.result[label]
                        });
                        total += data.result[label];
                    }

                    doPieChart(placeholder, dataset, total, formatter);

                    if (totalFormatter) {
                        // Put the label to box header
                        placeholder.parents('.box:first').find('.box-tools').prepend(
                            sprintf('<span class="badge bg-light-blue" title="%s">%s</span>', _('Total'), totalFormatter(total))
                        );
                    }
                } else {
                    displayError(placeholder, _('No data'));
                }
            } else {
                displayError(placeholder, data.message);
            }
        }).fail(function() {
            displayError(placeholder, _('Failed to communicate with server'));
        }).always(function() {
            placeholder.parents('.box:first').find('.overlay').remove();
        });
    });

    // Slice
    $('.chart-metrics.chart-lines').each(function(index, element) {
        var placeholder = $(element);
        var url = placeholder.data('url');
        var mode = placeholder.data('mode') || 'time';
        var converter = placeholder.data('converter') || 'value';
        var formatter = placeholder.data('formatter') || 'value';
        var total = 0;
        var timeFormat;

        placeholder.parents('.box:first').append('<div class="overlay"><i class="fa fa-circle-o-notch fa-spin fa-4x"> </i></div>');

        $.getJSON(url, wsParams, function(data) {
            if (data && data.status === true) {
                if (data.count > 0) {
                    if (data.step.months) {
                        timeFormat = (data.step.months == 1 && data.steps.length == 12) ? 'MMMM' : 'MM/YYYY';
                    }
                    else if (data.step.days) {
                        timeFormat = (data.step.days == 1 && data.steps.length == 7) ? 'dddd' : 'DD/MM';
                    }
                    else {
                        timeFormat = 'HH';
                    }
                    // Construct dataset
                    var dataset = [];
                    var labels = [];
                    for (var i=0; i<data.steps.length; i++) {
                        if (mode == 'time') {
                            labels.push(moment.utc(data.steps[i]).format(timeFormat));
                        } else {
                            labels.push(data.steps[i]);
                        }

                        dataset.push(converters[converter](data.result[i]));
                        total += data.result[i];
                    }

                    if (placeholder.data('total') === true) {
                        // Put the label to box header
                        placeholder.parents('.box:first').find('.box-tools').prepend(
                            sprintf('<span class="badge bg-light-blue" title="%s">%s</span>', _('Total'), formatters[formatter](total))
                        );
                    }

                    // Prepare data for chart
                    data = {
                        labels: labels,
                        datasets: [
                            {
                                fillColor: 'rgba(60, 141, 188, 0.5)',
                                strokeColor: 'rgba(60, 141, 188, 0.7)',
                                highlightFill: 'rgba(60, 141, 188, 0.9)',
                                highlightStroke: 'rgba(60, 141, 188, 1.0)',
                                data: dataset
                            }
                        ]
                    };

                    // Change formatter if needed
                    if (formatter == 'duration') {
                        if (converter == 'minutes') {
                            formatter = 'durationMinutes';
                        }
                        else if (converter == 'hours') {
                            formatter = 'durationHours';
                        }
                    }

                    // Go!
                    doBarChart(placeholder, data, total, formatter);

                } else {
                    displayError(placeholder, _('No data'));
                }
            }
        }).fail(function() {
            displayError(placeholder, _('Failed to communicate with server'));
        }).always(function() {
            placeholder.parents('.box:first').find('.overlay').remove();
        });
    });

    // Stats
    $('.stats-metrics').each(function(index, element) {
        var placeholder = $(element);
        var url = placeholder.data('url');

        $.getJSON(url, wsParams, function(data) {
            if (data && data.status === true) {
                $('.stats-value', placeholder).each(function(index, element) {
                    var name = $(element).data('name');
                    var formatter = formatters[$(element).data('formatter') || 'value'];
                    if (name in data) {
                        $(element).html(formatter(data[name]));
                    }
                });
            }
        }).fail(function() {
            displayError(placeholder, _('Failed to communicate with server'));
        });
    });

    // Tables
    $('.table-metrics').each(function(index, element) {
        var placeholder = $(element);
        var url = placeholder.data('url');

        placeholder.parents('.box:first').append('<div class="overlay"><i class="fa fa-circle-o-notch fa-spin fa-4x"> </i></div>');

        $.getJSON(url, wsParams, function(data) {
            if (data && data.status === true) {
                if (data.count) {
                    var table = $('<table class="table table-bordered table-hover"></table>').appendTo(placeholder);
                    var thead = $('<thead></thead>').appendTo(table);
                    var tbody = $('<tbody></tbody>').appendTo(table);
                    var tfoot = $('<tfoot></tfoot>').appendTo(table);
                    var value, header, th, tr;

                    // Add headers
                    tr = $('<tr></tr>').appendTo(thead);
                    for (var i=0; i<data.headers.length; i++) {
                        header = data.headers[i];
                        th = $('<th>' + header.label + '</th>').appendTo(tr);
                        if (header.type) {
                            th.data('type', 'numeric');
                        }
                    }

                    // Construct body
                    for (var i=0; i<data.result.length; i++) {
                        tr = $('<tr></tr>').appendTo(tbody);
                        for (var j=0; j<data.headers.length; j++) {
                            header = data.headers[j];
                            value = data.result[i][header.name];
                            if (header.type) {
                                if (header.type == 'duration') {
                                    tr.append('<td class="text-right" data-value="' + value + '">' + formatters.duration(value) + '</td>');
                                } else if (header.type == 'date') {
                                    tr.append('<td data-value="' + moment(value).unix() + '">' + moment(value).format('llll') + '</td>');
                                } else {
                                    // Default type
                                    tr.append('<td class="text-right">' + value + '</td>');
                                }
                            } else {
                                tr.append('<td>' + value + '</td>');
                            }
                        }
                    }

                    // Add pagination container
                    tr = $('<tr></tr>').appendTo(tfoot);
                    tr.append('<td class="text-right" colspan="' + data.headers.length + '"><ul class="pagination hide-if-no-paging"></ul></td>');

                    // FooTable!
                    table.footable({
                        filter: { input: placeholder.find('input.footable-filter') },
                        limitNavigation: 5,
                        pageSize: 20
                    });

                    // Put the label to box header
                    placeholder.parents('.box:first').find('.box-tools').prepend(
                        sprintf('<span class="badge bg-light-blue" title="%s">%s</span>', _('Total'), data.count)
                    );
                } else {
                    displayError(placeholder, _('No data'));
                }
            } else {
                displayError(placeholder, _('Failed to communicate with server'));
            }
        }).fail(function() {
            displayError(placeholder, _('Failed to communicate with server'));
        }).always(function() {
            placeholder.parents('.box:first').find('.overlay').remove();
        });
    });

    return true;
}


/*
 * CC-Metrics global initialization handler with semaphore
 */
CCMetricsInitHandler = function(value) {
    var callbacks = [];
    var lock = value || 1;

    return {
        done: function() {
            lock--;
            if (lock) {
                // Not ready
                return;
            }

            // Asynchronously calling callbacks
            for (var i=0; i<callbacks.length; i++) {
                setTimeout(callbacks[i], 100);
            }

            // Flush callbacks list
            callbacks = [];
        },

        ready: function(callback) {
            if (lock) {
                // Not ready, deferring call
                callbacks.push(callback);
                return;
            }

            // Asynchronously calling callback
            setTimeout(callback, 100);
        }
    }
};


function getDelta(range) {
    var delta = {
        years: range.until.year() - range.since.year(),
        months: range.until.month() - range.since.month(),
        days: range.until.date() - range.since.date()
    };

    // Handle M->M+1 transition
    if (delta.days < 0) {
        var daysInLastFullMonth = moment(range.until.year() + '-' + (range.until.month() + 1), "YYYY-MM").subtract(1, 'M').daysInMonth();
        if (daysInLastFullMonth < range.since.date()) { // 31/01 -> 2/03
            delta.days = daysInLastFullMonth + delta.days + (range.since.date() - daysInLastFullMonth);
        } else {
            delta.days = daysInLastFullMonth + delta.days;
        }
        delta.months--;
    }

    // Handle Y->Y+1 transition
    if (delta.months < 0) {
        delta.months = 12 + delta.months;
        delta.years--;
    }

    return delta;
}


function getRange() {
    var picker = $('#range-selector').data('daterangepicker');
    var since = picker.startDate.clone();
    var until = picker.endDate.clone();

    return {
        since: since,
        until: until
    }
}


function updateRangeLabel(since, until) {
    var label = null;
    for (var range in ranges) {
        if (since.isSame(ranges[range][0]) && until.isSame(ranges[range][1])) {
            label = range;
            break;
        }
    }
    if (!label) {
        label = since.format('L') + ' ' + until.format('L');
    }
    $('#range-selector span').html(label);
}

function refreshRange(since, until) {
    // Update hidden inputs
    $('#filters-since').val(since.startOf('day').toISOString());
    $('#filters-until').val(until.startOf('day').toISOString());

    var $form = $('form[name=filters]');
    $form.submit();
}


/*
 * Global initialization handler setup
 */
var ccmetrics = new CCMetricsInitHandler(1);

$(document).ready(function() {

    // Set Moment.js locale
    moment.locale(language);

    // bootstrap-select
    $('select').selectpicker();

    // Date range selector
    var since = $('#filters-since').val();
    var until = $('#filters-until').val();
    if (since && until) {
        since = moment.utc(since);
        until = moment.utc(until);
    }
    else {
        // Current month
        since = moment.utc().startOf('month');
        until = moment.utc().endOf('month');

        refreshRange(since, until);
    }

    // Custom ranges
    var today = moment.utc().startOf('day');

    ranges[_('Today')] = [today.clone(), today.clone()];
    ranges[_('Yesterday')] = [today.clone().subtract(1, 'days'), today.clone().subtract(1, 'days')];
    ranges[_('This Week')] = [today.clone().startOf('week'), today.clone().endOf('week').startOf('day')];
    ranges[_('This Month')] = [today.clone().startOf('month'), today.clone().endOf('month').startOf('day')];
    ranges[_('This Year')] = [today.clone().startOf('year'), today.clone().endOf('year').startOf('day')];
    ranges[_('Last Month')] = [today.clone().subtract(1, 'month').startOf('month'), today.clone().subtract(1, 'month').endOf('month').startOf('day')];
    ranges[_('Last Week')] = [today.clone().subtract(1, 'week').startOf('week'), today.clone().subtract(1, 'week').endOf('week').startOf('day')];
    ranges[_('Last Year')] = [today.clone().subtract(1, 'year').startOf('year'), today.clone().subtract(1, 'year').endOf('year').startOf('day')];
    ranges[_('Last 7 Days')] = [today.clone().subtract(6, 'days'), today.clone()];
    ranges[_('Last 30 Days')] = [today.clone().subtract(29, 'days'), today.clone()];

    updateRangeLabel(since, until);

    $('#range-selector').daterangepicker(
        {
            locale: {
                format: 'L',
                applyLabel: _('Apply'),
                cancelLabel: _('Cancel'),
                fromLabel: _('From'),
                toLabel: _('To'),
                customRangeLabel: _('Custom')
            },
            ranges: ranges,
            startDate: since,
            endDate: until,
            showDropdowns: true
        },
        refreshRange
    );

    // Actions
    $('#next-range').on('click', function(e) {
        e.preventDefault();

        var range = getRange();
        range.until.add(1, 'day');
        var delta = getDelta(range);

        range.since.add(delta);
        range.until.add(delta).subtract(1, 'day');

        $('#range-selector').data('daterangepicker').setEndDate(range.until);
        $('#range-selector').data('daterangepicker').setStartDate(range.since);

        refreshRange(range.since, range.until);
    });

    $('#prev-range').on('click', function(e) {
        e.preventDefault();

        var range = getRange();
        range.until.add(1, 'day');
        var delta = getDelta(range);

        range.since.subtract(delta);
        range.until.subtract(delta).subtract(1, 'day');

        $('#range-selector').data('daterangepicker').setStartDate(range.since);
        $('#range-selector').data('daterangepicker').setEndDate(range.until);

        refreshRange(range.since, range.until);
    });

    $('#refresh').on('click', function(e) {
        e.preventDefault();

        var $form = $('form[name=filters]');
        $form.submit();
    });

    // Resize graphics when toggling sidebar
    $('body').on('webkitTransitionEnd otransitionend oTransitionEnd msTransitionEnd transitionend', function(event) {
        $('.chart-metrics').each(function(index, element) {
            var chart = $(element).data('chart');
            if (chart.options.responsive) {
                chart.resize(chart.render, true);
            }
        });
    });

    // Scroll-top
    $(window).scroll(function() {
        var $scrollTop = $('#scroll-top a');
        if ($(this).scrollTop() > 100) {
            if (!$scrollTop.hasClass('active')) {
                $('#scroll-top a').fadeIn('fast').addClass('active');
            }
        } else {
            if ($scrollTop.hasClass('active')) {
                $('#scroll-top a').fadeOut('fast').removeClass('active');
            }
        }
    }).trigger('scroll');

    // CC-Metrics is ready
    ccmetrics.done();
});
